Library IEEE;
USE IEEE.std_logic_1164.all;
USE IEEE.std_logic_unsigned.all;

-- My Definitions --------------------------------
USE work.pkg_081006.all; 

------------------------------------------------------
-- Version 2	This version uses a different curve for the Filter2.
--					This curve ramps down and then back up.  The goal
--					is to mask off any glitch from the hold switch, but
-- 				then allow the ramp to the final offset to procede.


entity BW_Curve_Generator_v2 is
	port(
			clk					: in std_logic;
			nreset				: in std_logic;
			Filter_Trigger		: in std_logic;
			
			First_Point_us		: in std_logic_vector(15 downto 0);
			Final_Point_us		: in std_logic_vector(15 downto 0);
			intv_clks  			: in std_logic_vector(15 downto 0);
			Update_Strb 		: in std_logic;
			Cycle_Reset			: in std_logic;
			
			BW_Curve_Out_us	: out std_logic_vector(15 downto 0);
			Curve_Strobe 		: out std_logic;
			
			go_out				: out std_logic
			    
		);
end BW_Curve_Generator_v2;

architecture main of BW_Curve_Generator_v2 is

	signal go 			: std_logic;

------------------------------------------------------------
-- ALTERA COMPONENTS ---------------------------------------
-- The following components are the ROM memory and a multiplier
-- They were created using the LPM Megafunction Wizard.
-- The following VHDL files were generated and must be included 
-- in the project.
--							  curve_mem2.vhd
--                     addsub16unsigned
--                     mult16unsigned.vhd
--
-- The Megafunction Wizard also generate *.cmp files for each
-- from which I cut and pasted the component declarations below.
--
----------------------------------------------------------------

component curve_mem_dnup
	PORT
	(
		address	: IN STD_LOGIC_VECTOR (9 DOWNTO 0);
		clock		: IN STD_LOGIC ;
		data		: IN STD_LOGIC_VECTOR (15 DOWNTO 0);
		wren		: IN STD_LOGIC ;
		q			: OUT STD_LOGIC_VECTOR (15 DOWNTO 0)
	);
end component;

-- Curve Generation --------------------------------
	signal curve_us			: std_logic_vector(15 downto 0);
	signal  r1_us				: std_logic_vector(15 downto 0);
	signal  product_us		: std_logic_vector(31 downto 0);
	signal curve_out_us		: std_logic_vector(15 downto 0);
	signal curve_latch_us	: std_logic_vector(15 downto 0);
	
	
-----------------------------------------------------------------
component mult16unsigned
	PORT
	(
		clock		: IN STD_LOGIC ;
		dataa		: IN STD_LOGIC_VECTOR (15 DOWNTO 0);
		datab		: IN STD_LOGIC_VECTOR (15 DOWNTO 0);
		result		: OUT STD_LOGIC_VECTOR (31 DOWNTO 0)
	);
end component;

	
	
component addsub16unsigned
	PORT
	(
		add_sub	: IN STD_LOGIC ;
		dataa		: IN STD_LOGIC_VECTOR (15 DOWNTO 0);
		datab		: IN STD_LOGIC_VECTOR (15 DOWNTO 0);
		result	: OUT STD_LOGIC_VECTOR (15 DOWNTO 0)
	);
end component;

-- ADDR_CNT Counter/Timer declarations ---------------------------
-- The output of this counter is the address pointer
	signal nclr_adr : std_logic;  -- output of sm
	signal ld_adr	 : std_logic;
	signal inc_adr  : std_logic;  -- output of sm
	signal last_addr_flg : std_logic;  -- output of ADDR_CNT, input to sm
	signal last_adr : std_logic_vector(9 downto 0);  -- input to ADDR_CNT
	signal addr     : std_logic_vector(9 downto 0);  -- output of ADDR_CNT
	
	-- INTERVAL_CNT declarations ------------------------------
	signal nclr_cnt : std_logic; -- output of sm
	signal smp_flag : std_logic; -- output of INTERVAL_CNT, input to sm
	
	signal strb_delay	: std_logic_vector(1 downto 0);
	
	-- State Machine declarations ------------------------------------
	type STATE_TYPE is (idle, wait_1, nextadr, hold);
	signal CS, NS  : STATE_TYPE;


--%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
--**********************************************************
begin
	sync_START: led
	port map(sig_in => Filter_Trigger, clk => clk, clrn => nreset, pls_out => go);	
	
	
--oooooooooooooooooooooooooooooooooooooooooooooooooooooooooo
-- Curve Memory --------------------------------------------
	CURVE_MEMORY: curve_mem_dnup
	port map(address => addr, 
				clock => clk, 
				data => (others => '0'),
				wren => '0',
				q => curve_us(15 downto 0)
			);
						
	FIRST_MINUS_FINAL: addsub16unsigned
	port map (
				add_sub	 => '0', -- subtraction, A-B
				dataa	 => First_Point_us, -- Curve is expected to fall from first to final
				datab	 => Final_Point_us,
				result => r1_us(15 downto 0)
			 );

-- Composite Curve Computation ---------------------------------------	
	R1_x_CURVE: mult16unsigned 
	port map (
				clock	 => clk,
				dataa	 => r1_us(15 downto 0),
				datab	 => curve_us(15 downto 0),
				result => product_us(31 downto 0)
			 );
			 			 

	PRODUCT_PLUS_FINAL: addsub16unsigned
	port map (
				add_sub	=> '1', -- subtraction, A+B
				dataa	 	=> product_us(31 downto 16),
				datab	 	=> Final_Point_us,
				result 	=> curve_out_us(15 downto 0)
			 );

	LATCH_CURVE: process(clk, nreset, curve_out_us, Update_Strb)
	begin
		if   (nreset = '0') 	      then curve_latch_us <= (others => '0');
		elsif rising_edge(clk) 	then
			if (Update_Strb = '1') then 	curve_latch_us <= curve_out_us;
			end if;
		end if;
	end process;

	BW_Curve_Out_us <= curve_latch_us(15 downto 0);
	
--------------------------------------------------------------
--Memory Address Pointer -------------------------------------		
	ADDR_CNT: counter  -- address sequencer
	-- The output of this counter is the address pointer
	generic map(10)
	port map ( clk=>clk, clrn=>'1', ena=>inc_adr, load=>ld_adr, updn=>'1',
			   ld_data  => "0000000000",
			   flag_cnt => last_adr,
			   flag_out => last_addr_flg,
			   cnt_out  => addr
			 );
	last_adr <= "1111111110"; 

--------------------------------------------------------------
	INTERVAL_CNT: counter  -- Defines the curve sample interval
	generic map(WID16)
	port map ( clk=>clk, clrn=>nclr_cnt, ena=>'1', load=>'0', updn=>'1',
			   ld_data  => (others => '0'),
			   flag_cnt => intv_clks,
			   flag_out => smp_flag,
			   cnt_out  => open
			 );

---------------------------------------------------------------		
-- State Machine Process definition ---------------------------
	sm_reg: process (clk, nreset)	
	begin
		if(nreset='0')              then CS <= idle;
		elsif(clk'event and clk='1')then CS <= NS;
		end if;
	end process; -- sm_reg
	
	sm_combinatorial: 
	process (CS, smp_flag, last_addr_flg, Cycle_Reset, go)
	begin
		nclr_cnt <= '1'; -- counter is reset and new interval adjustment is latched.
		inc_adr  <= '0'; -- not incrementing the address pointer
		ld_adr	<= '0';
		
		go_out	<= '0';
		
		case CS is
			when idle   =>
				ld_adr	<= '1';
				nclr_cnt <= '0'; -- interval clocks counter is reset
				if(go ='1') then 					NS <= wait_1;
				else                      		NS <= idle;
				end if; 
			
			when wait_1 => -- wait for next sample time
				if   (Cycle_Reset ='1')then 	NS <= idle;
				elsif(smp_flag    ='1')then 	NS <= nextadr;
				else                      		NS <= wait_1;
				end if;
		go_out	<= '1';
				
			when nextadr => 
				if   (Cycle_Reset ='1')then 		NS <= idle;
				elsif(last_addr_flg ='1') then 	NS <= hold;
				else                      			NS <= wait_1;
				end if;
				-- outputs --
				nclr_cnt <= '0'; -- clear the interval count
				inc_adr  <= '1';  -- increment the addr count
				
			when hold => -- Hold last address until end of cycle trigger
				if(Cycle_Reset ='1') then 		NS <= idle;
				else                      		NS <= hold;
				end if;
				
		end case; 
				
	end process; -- sm_combinatorial

-- Update Strobe Out ---------------------------------------------------
	STROBE_DELAY: 
	process (clk, nreset, Update_Strb, strb_delay)
	begin
		if   (nreset = '0') then strb_delay <= (others => '0');
										 Curve_Strobe <= '0';

		elsif rising_edge(clk) then
			strb_delay(0) <= Update_Strb;
			strb_delay(1) <= strb_delay(0);
			Curve_Strobe <= strb_delay(1);
		end if;
	end process;

	
end;
