Library IEEE;
USE IEEE.std_logic_1164.all;
USE IEEE.std_logic_unsigned.all;

-- My Definitions --------------------------------
USE work.pkg_081006.all; 

------------------------------------------------------

entity DPOT_Control_Mux_v2 is
	port(
			clk			: in std_logic;
			nreset		: in std_logic;
			DPOT_Addr	: in std_logic_vector(3 downto 0);
			cs_strobe	: in std_logic;
			step_pulse	: in std_logic;

			ADC1_Adj_UDn 	:out std_logic;
			ADC2_Adj_UDn 	: out std_logic;
			DAC_Adj_UDn	 	: out std_logic;
			
			ADC_OFFS_CS1n : out std_logic;
			ADC_GAIN_CS1n : out std_logic;
			
			ADC_OFFS_CS2n : out std_logic;
			ADC_GAIN_CS2n : out std_logic;
               
			DAC_OFFS_CS1n : out std_logic;
			DAC_GAIN_CS1n : out std_logic;
			
			DAC_OFFS_CS2n : out std_logic;
			DAC_GAIN_CS2n : out std_logic
						    
		);
end DPOT_Control_Mux_v2;

architecture main of DPOT_Control_Mux_v2 is

signal cs_n	: std_logic;

signal set_test_mode 	: std_logic;
signal clr_test_mode 	: std_logic;
signal set_test_pos  	: std_logic;
signal set_test_neg  	: std_logic;

--========================================================
--                  B E G I N
--========================================================
begin

	cs_n <= cs_strobe;

process(DPOT_Addr, cs_n, step_pulse)
begin
	-- DEFAULT VALUES ----------------
	ADC_OFFS_CS1n <= '1';
	ADC_OFFS_CS2n <= '1';
	DAC_OFFS_CS1n <= '1';
	DAC_OFFS_CS2n <= '1';

	ADC_GAIN_CS1n <= '1';
	ADC_GAIN_CS2n <= '1';
	DAC_GAIN_CS1n <= '1';
	DAC_GAIN_CS2n <= '1';

	ADC1_Adj_UDn 	<= '0';
	ADC2_Adj_UDn 	<= '0'; 	
	DAC_Adj_UDn 	<= '0';	 
	

	case DPOT_Addr is	
		-- Adjusting ADC's --
		-------------------------------------------------
		when "0000" => 	ADC_OFFS_CS1n 	<= cs_n;
								ADC1_Adj_UDn	<= step_pulse;
								
		when "0001" => 	ADC_GAIN_CS1n 	<= cs_n;
								ADC1_Adj_UDn	<= step_pulse;
		
		-------------------------------------------------
		when "0010" => 	ADC_OFFS_CS2n 	<= cs_n;
								ADC2_Adj_UDn	<= step_pulse;
								
		when "0011" => 	ADC_GAIN_CS2n 	<= cs_n;
								ADC2_Adj_UDn	<= step_pulse;
		
		-- Adjusting DAC's --
		-------------------------------------------------
		when "1000" => 	DAC_OFFS_CS1n 	<= cs_n;
								DAC_Adj_UDn		<= step_pulse;
								
		when "1001" => 	DAC_GAIN_CS1n 	<= cs_n;
								DAC_Adj_UDn		<= step_pulse;
		-------------------------------------------------
		when "1010" => 	DAC_OFFS_CS2n 	<= cs_n;
								DAC_Adj_UDn		<= step_pulse;
								
		when "1011" => 	DAC_GAIN_CS2n 	<= cs_n;
								DAC_Adj_UDn		<= step_pulse;
					
		when others =>  NULL;
	end case;
end process;
   		

end;----------------------------------------------------------