Library IEEE;
USE IEEE.std_logic_1164.all;
USE IEEE.std_logic_unsigned.all;

-- My Definitions --------------------------------
USE work.pkg_081006.all; 

------------------------------------------------------

entity DPOT_Control_Mux_v3 is
	port(
			clk			: in std_logic;
			nreset		: in std_logic;
			DPOT_Addr	: in std_logic_vector(3 downto 0);
			cs_strobe	: in std_logic;
			step_pulse	: in std_logic;

			RSSI_HIGH_UDn 	: out std_logic;
			ADC_ADJ_UDn 	:out std_logic;
			DPOT_U_nD	 	: out std_logic;
			
			RSSI_HIGH_CSn 	: out std_logic;
			
			OFFADJ_CSn 		: out std_logic;
			GAINADJ_CSn 	: out std_logic;
			
			DAC_1_OFFS_CSn : out std_logic;
			DAC_1_GAIN_CSn : out std_logic;
               
			DAC_2_OFFS_CSn : out std_logic;
			DAC_2_GAIN_CSn : out std_logic
               
						    
		);
end DPOT_Control_Mux_v3;

architecture main of DPOT_Control_Mux_v3 is

signal cs_n	: std_logic;

signal set_test_mode 	: std_logic;
signal clr_test_mode 	: std_logic;
signal set_test_pos  	: std_logic;
signal set_test_neg  	: std_logic;

--========================================================
--                  B E G I N
--========================================================
begin

	cs_n <= cs_strobe;

process(DPOT_Addr, cs_n, step_pulse)
begin
	-- DEFAULT VALUES ----------------
			RSSI_HIGH_CSn 	<= '1';
			
			OFFADJ_CSn 		<= '1';
			GAINADJ_CSn 	<= '1';
			
			DAC_1_OFFS_CSn <= '1';
			DAC_1_GAIN_CSn <= '1';
               
			DAC_2_OFFS_CSn <= '1';
			DAC_2_GAIN_CSn <= '1';

			RSSI_HIGH_UDn 	<= '0';
			ADC_ADJ_UDn 	<= '0';
			DPOT_U_nD	 	<= '0';
	

	case DPOT_Addr is	
		-- RSSI HIGH THRESHOLD --
		-------------------------------------------------
		when "0001" => 	RSSI_HIGH_CSn 	<= cs_n;
								RSSI_HIGH_UDn	<= step_pulse;
		
		-- Adjusting The ADC --
		-------------------------------------------------
		when "0010" => 	OFFADJ_CSn 		<= cs_n;
								ADC_ADJ_UDn		<= step_pulse;
								
		when "0011" => 	GAINADJ_CSn 	<= cs_n;
								ADC_ADJ_UDn		<= step_pulse;
		
		-- Adjusting DAC's --
		-------------------------------------------------
		when "0100" => 	DAC_1_OFFS_CSn <= cs_n;
								DPOT_U_nD		<= step_pulse;
								
		when "0101" => 	DAC_1_GAIN_CSn <= cs_n;
								DPOT_U_nD		<= step_pulse;
		-------------------------------------------------
		when "0110" => 	DAC_2_OFFS_CSn <= cs_n;
								DPOT_U_nD		<= step_pulse;
								
		when "0111" => 	DAC_2_GAIN_CSn <= cs_n;
								DPOT_U_nD		<= step_pulse;
					
		when others =>  NULL;
	end case;
end process;
   		

end;----------------------------------------------------------