Library IEEE;
USE IEEE.std_logic_1164.all;
USE IEEE.std_logic_unsigned.all;

-- My Definitions --------------------------------
USE work.pkg_081006.all; 

------------------------------------------------------

entity Encoder_Interface is
	port(
			clk			: in std_logic;
			reset_n		: in std_logic;
			Encoder_A	: in std_logic;
			Encoder_B	: in std_logic;
			
			Direction	: out std_logic;
			Step		: out std_logic
		);
end Encoder_Interface;

architecture main of Encoder_Interface is

-- Signals to syncronize the inputs --------------------------------------
signal syncA		: std_logic_vector(1 downto 0);
signal syncB		: std_logic_vector(1 downto 0);

signal BA			: std_logic_vector(1 downto 0);

signal dir_reg		: std_logic;

-- ENC_STATE_MACHINE --------------------------------------
type STATE_TYPE is
		(get_BA, BA_00, BA_01, BA_10, BA_11, set_inc, set_dec, do_step);
	signal CS, NS : STATE_TYPE;	

--========================================================
--                  B E G I N
--========================================================
begin

SYNCRONIZE_INPUTS: process(clk, reset_n, Encoder_A, Encoder_B)
begin
	if    (reset_n = '0')    then	
			syncA <= "00";
			syncB <= "00";
	elsif (rising_edge(clk)) then
			syncA(0) <= Encoder_A;
			syncA(1) <= syncA(0);
			
			syncB(0) <= Encoder_B;
			syncB(1) <= syncB(0);
			
			BA <= syncB(1) & syncA(1);
	end if;
end process;

--=======================================================================
-- STATE MACHINE  
--=======================================================================
SM_REG: 
process(clk, reset_n)
begin
	if(reset_n = '0')		then CS <= get_BA;
	elsif(rising_edge(clk))	then CS <= NS;
	end if;
end process;

SM_NEXT_STATE: 
process(CS, BA)
begin
	-- DEFAULT VALUES ----------------
	Step <= '0';
	
	case CS is
		when get_BA => 
			case BA is
				when "00" =>		NS <= BA_00;
				when "01" =>		NS <= BA_01;
				when "10" =>		NS <= BA_10;
				when "11" =>		NS <= BA_11;
				when others =>		NS <= get_BA;
			end case;
			
		when BA_00 =>
			case BA is
				when "00" =>		NS <= BA_00;
				when "01" =>		NS <= set_inc;
				when "10" =>		NS <= set_dec;
				when "11" =>		NS <= BA_11;
				when others =>		NS <= get_BA;
			end case;

		when BA_01 =>
			case BA is
				when "00" =>		NS <= set_dec;
				when "01" =>		NS <= BA_01;
				when "10" =>		NS <= BA_10;
				when "11" =>		NS <= set_inc;
				when others =>		NS <= get_BA;
			end case;

		when BA_10 =>
			case BA is
				when "00" =>		NS <= set_inc;
				when "01" =>		NS <= BA_01;
				when "10" =>		NS <= BA_10;
				when "11" =>		NS <= set_dec;
				when others =>		NS <= get_BA;
			end case;

		when BA_11 =>
			case BA is
				when "00" =>		NS <= BA_00;
				when "01" =>		NS <= set_dec;
				when "10" =>		NS <= set_inc;
				when "11" =>		NS <= BA_11;
				when others =>		NS <= get_BA;
			end case;
		
		when set_inc =>				NS <= do_step;
			-- see DIRECTION_REGISTER below
		
		when set_dec =>				NS <= do_step;
			-- see DIRECTION_REGISTER below
		
		when do_step =>				NS <= get_BA;
			Step <= '1';
								  
		when others =>              NS <= get_BA;
	end case;
end process;

DIRECTION_REGISTER: 
process(clk, reset_n, CS)
begin
	if(reset_n = '0')		 then dir_reg <= '0';
	elsif(rising_edge(clk))	 then 
		if    (CS = set_inc) then dir_reg <= '1';
		elsif (CS = set_dec) then dir_reg <= '0';
		end if;
	end if;
end process;
Direction <= dir_reg;

end;----------------------------------------------------------