Library IEEE;
USE IEEE.std_logic_1164.all;
USE IEEE.std_logic_arith.all;
USE ieee.std_logic_unsigned.all;

-- My Definitions --------------------------------------------------
USE work.pkg_081006.all;  -- my definitions and entities

--------------------------------------------------------------------
-- When the Booster_Test_Event rising edge is detected the Beam_Gate
-- is replaced by the Accel_Gate.  When there is a Booster cycle
-- without beam we will not get a Beam_Gate, but we will always
-- get the Accel_Gate.

entity No_Beam_Test_Logic is
	port(clk        	: in std_logic;
		 nreset     	: in std_logic;
		 
		 Beam_Gate			: in std_logic;
		 Accel_Gate			: in std_logic;
		 Booster_Test_Event	: in std_logic;
		 
 		 Test_Beam_Gate		: out std_logic;

		 Enable_Offset		: out std_logic
		);
end No_Beam_Test_Logic;
	
architecture main of No_Beam_Test_Logic is

signal syncd_Accel_Gate	: std_logic;
signal syncd_Beam_Gate 	: std_logic;

signal do_test			: std_logic;
signal enable_test		: std_logic;

signal timer		: std_logic_vector(23 downto 0);
signal timer_ena	: std_logic;


	-- State Machine declarations ------------------------------------
	type STATE_TYPE is 
	(idle, test_cycle, done);
	signal CS, NS  : STATE_TYPE;

-- ## MAIN ##############################################################
begin 

sync_Booster_Test_Event: led
port map(sig_in => Booster_Test_Event, clk => clk, clrn => nreset, pls_out => do_test);


process(clk, nreset, Beam_Gate, Accel_Gate)
begin
	if (nreset = '0') then 
		syncd_Accel_Gate	<= '0';
		syncd_Beam_Gate		<= '0';
	elsif rising_edge(clk) then
		syncd_Accel_Gate	<= Accel_Gate;
		syncd_Beam_Gate		<= Beam_Gate;
	end if;
end process;

PROCESS_TIMER: process(clk, nreset, timer_ena, timer)
begin
	if  (nreset = '0') then 	timer <= (others => '0');
	elsif rising_edge(clk) then
		if  (timer_ena = '0') then 	timer <= (others => '0');
		else						timer <= timer + 1;
		end if;
	end if;
end process;

----------------------------------------------------------
TEST_BEAM_GATE_OUTPUT:
process(enable_test, syncd_Accel_Gate, syncd_Beam_Gate)
begin
	if  (enable_test = '1') then 	Test_Beam_Gate <= syncd_Accel_Gate;
									Enable_Offset  <= '1';
	else							Test_Beam_Gate <= syncd_Beam_Gate;
									Enable_Offset  <= '0';
	end if;
end process;

----------------------------------------------------------	
sm_reg: process (clk, nreset)	
begin
	if(nreset='0')              then CS <= idle;
	elsif(clk'event and clk='1')then CS <= NS;
	end if;
end process; -- sm_reg

sm_combinatorial: 
process (CS, timer, do_test)
begin
	-- default outputs --
	timer_ena 	<= '1'; -- not enabled only when in idle state
	enable_test	<= '0';	
			
	case CS is
		when idle   =>
			timer_ena		<= '0';   			
			if (do_test = '1') then 		NS <= test_cycle;	
			else							NS <= idle;
			end if;
	
		when test_cycle  =>
			enable_test	<= '1';	
			if(timer = X"30D400")   then	NS <= done; -- stay here for 40 ms
			else							NS <= test_cycle;
			end if;
			
		when done  =>						NS <= idle;
					
		when others =>						NS <= idle;
	end case; 
			
end process;

end main;