Library IEEE;
USE IEEE.std_logic_1164.all;
USE IEEE.std_logic_unsigned.all;

-------------------------------------------------------------------------------
-- MODULE DESCRIPTION, March 17, 2010
--
-- This is where the address bits are decoded for the Lower FPGA memory map.
-- If 


entity Parameter_Registers_v2 is 
	port( 	
			clk					: in std_logic;
			nreset				: in std_logic;
			
			Data_to_Mem 	: in std_logic_vector(15 downto 0);
			Addr_to_Mem		: in std_logic_vector(7 downto 0);
			Mem_WR			: in std_logic;	

			Param_Data_Out	: out std_logic_vector(15 downto 0);
			Param_0			: out std_logic_vector(15 downto 0);
			Param_1			: out std_logic_vector(15 downto 0);
			Param_2			: out std_logic_vector(15 downto 0);
			Param_3			: out std_logic_vector(15 downto 0);
			Param_4			: out std_logic_vector(15 downto 0);
			Param_5			: out std_logic_vector(15 downto 0);
			Param_6			: out std_logic_vector(15 downto 0);
			Param_7			: out std_logic_vector(15 downto 0);
			Param_8			: out std_logic_vector(15 downto 0);
			Param_9			: out std_logic_vector(15 downto 0);
			Param_A			: out std_logic_vector(15 downto 0);
			Param_B			: out std_logic_vector(15 downto 0);
			Param_C			: out std_logic_vector(15 downto 0);
			Param_D			: out std_logic_vector(15 downto 0);
			Param_E			: out std_logic_vector(15 downto 0);
			Param_F			: out std_logic_vector(15 downto 0);
			updated			: out std_logic
			        
		);
	end Parameter_Registers_v2;
	
architecture main of Parameter_Registers_v2 is	

signal ADDR 		: std_logic_vector(7 downto 0);
signal DATA			: std_logic_vector(15 downto 0);
signal WR			: std_logic;

signal param_0_out	: std_logic_vector(15 downto 0);
signal param_1_out	: std_logic_vector(15 downto 0);
signal param_2_out	: std_logic_vector(15 downto 0);
signal param_3_out	: std_logic_vector(15 downto 0);
signal param_4_out	: std_logic_vector(15 downto 0);
signal param_5_out	: std_logic_vector(15 downto 0);
signal param_6_out	: std_logic_vector(15 downto 0);
signal param_7_out	: std_logic_vector(15 downto 0);
signal param_8_out	: std_logic_vector(15 downto 0);
signal param_9_out	: std_logic_vector(15 downto 0);
signal param_A_out	: std_logic_vector(15 downto 0);
signal param_B_out	: std_logic_vector(15 downto 0);
signal param_C_out	: std_logic_vector(15 downto 0);
signal param_D_out	: std_logic_vector(15 downto 0);
signal param_E_out	: std_logic_vector(15 downto 0);
signal param_F_out	: std_logic_vector(15 downto 0);

signal sync_upd		: std_logic_vector(2 downto 0);
             
begin

	DATA <= Data_to_Mem; 
	ADDR <=	Addr_to_Mem;	
	WR	 <= Mem_WR;
    
	PARAMETER_REGISTERS: process(clk, nreset, WR, ADDR, DATA)
	begin	
		if rising_edge(clk) then
			if (nreset = '0') then
				param_0_out <= (others => '0');
				param_1_out <= (others => '0');
				param_2_out <= (others => '0');
				param_3_out <= (others => '0');
				param_4_out <= (others => '0');
				param_5_out <= (others => '0');
				param_6_out <= (others => '0');
				param_7_out <= (others => '0');
				param_8_out <= (others => '0');
				param_9_out <= (others => '0');
				param_A_out <= (others => '0');
				param_B_out <= (others => '0');
				param_C_out <= (others => '0');
				param_D_out <= (others => '0');
				param_E_out <= (others => '0');
				param_F_out <= (others => '0');
			elsif(WR = '1') then
				case ADDR is
					when X"00" => param_0_out <= DATA;
					when X"02" => param_1_out <= DATA;
					when X"04" => param_2_out <= DATA;
					when X"06" => param_3_out <= DATA;
					when X"08" => param_4_out <= DATA;
					when X"0A" => param_5_out <= DATA;
					when X"0C" => param_6_out <= DATA;
					when X"0E" => param_7_out <= DATA;
					when X"10" => param_8_out <= DATA;
					when X"12" => param_9_out <= DATA;
					when X"14" => param_A_out <= DATA;
					when X"16" => param_B_out <= DATA;
					when X"18" => param_C_out <= DATA;
					when X"1A" => param_D_out <= DATA;
					when X"1C" => param_E_out <= DATA;
					when X"1E" => param_F_out <= DATA;	
					when others => NULL;			                      
				end case;
			end if;
		end if;
	end process;
       
	DATA_OUT_MUX: 
	process(ADDR, param_0_out, param_1_out, param_2_out, param_3_out, 
				  param_4_out, param_5_out, param_6_out, param_7_out,
				  param_8_out, param_9_out, param_A_out, param_B_out,
				  param_C_out, param_D_out, param_E_out, param_F_out)
	begin
		case ADDR is
			when X"00" => Param_Data_Out <= param_0_out;
			when X"02" => Param_Data_Out <= param_1_out;
			when X"04" => Param_Data_Out <= param_2_out;
			when X"06" => Param_Data_Out <= param_3_out;
			when X"08" => Param_Data_Out <= param_4_out;
			when X"0A" => Param_Data_Out <= param_5_out;
			when X"0C" => Param_Data_Out <= param_6_out;
			when X"0E" => Param_Data_Out <= param_7_out;
			when X"10" => Param_Data_Out <= param_8_out;
			when X"12" => Param_Data_Out <= param_9_out;
			when X"14" => Param_Data_Out <= param_A_out;
			when X"16" => Param_Data_Out <= param_B_out;
			when X"18" => Param_Data_Out <= param_C_out;
			when X"1A" => Param_Data_Out <= param_D_out;
			when X"1C" => Param_Data_Out <= param_E_out;
			when X"1E" => Param_Data_Out <= param_F_out;				 				  
			when others => Param_Data_Out <= param_0_out;			                      
		end case;                                 
				
	end process;

	Param_0 <= param_0_out;
	Param_1 <= param_1_out;
	Param_2 <= param_2_out;
	Param_3 <= param_3_out;
	Param_4 <= param_4_out;
	Param_5 <= param_5_out;
	Param_6 <= param_6_out;
	Param_7 <= param_7_out;
	Param_8 <= param_8_out;
	Param_9 <= param_9_out;
	Param_A <= param_A_out;
	Param_B <= param_B_out;
	Param_C <= param_C_out;
	Param_D <= param_D_out;
	Param_E <= param_E_out;
	Param_F <= param_F_out;
	                
	UPDATED_PARAM_INDICATION:
	process(clk, nreset, WR, sync_upd)
    begin
		if (nreset = '0') then sync_upd <= (others => '0');
		elsif rising_edge(clk) then
			sync_upd(0) <= WR;
			sync_upd(1) <= sync_upd(0);
			sync_upd(2) <= sync_upd(1);
		end if;
		
		updated	<= sync_upd(1) AND NOT sync_upd(2);
    end process;   
end;