--------------------------------------------------------------------


Library IEEE;
USE IEEE.std_logic_1164.all;
USE IEEE.std_logic_arith.all;
USE ieee.std_logic_unsigned.all;

-- My Definitions --------------------------------------------------
USE work.pkg_081006.all;  -- my definitions and entities

entity Running_Sum_v1 is
	port(clk        			: in std_logic;
		 nreset     			: in std_logic;
		 Data_In 				: in std_logic_vector(15 downto 0);
		 Data_Strb_In			: in std_logic;
		 Buffer_Length			: in std_logic_vector(15 downto 0);
		 
		 Sum_Out 				: out std_logic_vector(15 downto 0)
		);
end Running_Sum_v1;
	
architecture main of Running_Sum_v1 is

------------------------------------------------------------
-- ALTERA COMPONENTS ---------------------------------------
-- The following components are the ROM memory and a multiplier
-- They were created using the LPM Megafunction Wizard.
-- The following VHDL files were generated and must be included 
-- in the project.
--                              curve_mem1.vhd
--                            mult12signed.vhd
--                            mult14signed.vhd
--
-- The Megafunction Wizard also generate *.cmp files for each
-- from which I cut and pasted the component declarations below.
----------------------------------------------------------------
	component curve_mem1
		PORT
		(
			address	: IN STD_LOGIC_VECTOR (7 DOWNTO 0);
			clock		: IN STD_LOGIC ;
			data		: IN STD_LOGIC_VECTOR (11 DOWNTO 0);
			wren		: IN STD_LOGIC ;
			q			: OUT STD_LOGIC_VECTOR (11 DOWNTO 0)
		);
	end component;

	component mult12signed
		PORT
		(
			clock		: IN STD_LOGIC ;
			dataa		: IN STD_LOGIC_VECTOR (11 DOWNTO 0);
			datab		: IN STD_LOGIC_VECTOR (11 DOWNTO 0);
			result		: OUT STD_LOGIC_VECTOR (23 DOWNTO 0)
		);
	end component;

-----------------------------------------------------------------

	-- ADDR_CNT Counter/Timer declarations ---------------------------
	-- The output of this counter is the address pointer
	signal nclr_adr : std_logic;  -- output of sm
	signal inc_adr  : std_logic;  -- output of sm
	signal end_flag : std_logic;  -- output of ADDR_CNT, input to sm
	signal last_adr : std_logic_vector(7 downto 0);  -- input to ADDR_CNT
	signal addr     : std_logic_vector(7 downto 0);  -- output of ADDR_CNT
	
	-- INTERVAL_CNT declarations ------------------------------
	signal nclr_cnt : std_logic; -- output of sm
	signal smp_flag : std_logic; -- output of INTERVAL_CNT, input to sm

	-- Sync signal declarations --------------------------------------
	signal go        : std_logic;  -- input to sm
	signal end_cycle : std_logic; -- input to sm
	
	signal curve_active : std_logic;

	-- State Machine declarations ------------------------------------
	type STATE_TYPE is 
	(init, idle, wait_1, nextadr, hold, rst_adr);
	signal CS, NS  	: STATE_TYPE;
	
	-- ROM Curve intermediate signals -------------------------------
	signal curve   	: std_logic_vector(11 downto 0);
	signal curve_0		: std_logic_vector(11 downto 0);
	
	-- Multiplier intermediate signals ------------------------------
	signal base_gain_2s : std_logic_vector(11 downto 0);
	signal curve_2s     : std_logic_vector(11 downto 0);
	signal product_2s   : std_logic_vector(23 downto 0);
	
	signal new_gain     : std_logic_vector(11 downto 0);

-- ## MAIN #########################################################
begin 
	-- convert 12 bit offset binary to 12 bit 2's comlement
	base_gain_2s <= not base_gain(11) & base_gain(10 downto 0);
	
------------------------------------------------------------
-- ALTERA COMPONENTS ---------------------------------------
	CURVE_MEM: curve_mem1
	port map(address => addr, 
			 clock => clk, 
			 data => (others => '0'),
			 wren => '0',
			 q => curve_0(11 downto 0)
			);
			
	curve_2s <= (not curve_0(11) & curve_0(10 downto 0));		
		
	G1xCURVE : mult12signed 
	port map (
				clock	 => clk,
				dataa	 => base_gain_2s,
				datab	 => curve_2s,
				result => product_2s
			 );
			
-------------------------------------------------------------	
		
	ADDR_CNT: counter  -- address sequencer
	-- The output of this counter is the address pointer
	generic map(WID8)
	port map ( clk=>clk, clrn=>nclr_adr, ena=>inc_adr, load=>'0', updn=>'1',
			   ld_data  => (others => '0'),
			   flag_cnt => last_adr,
			   flag_out => end_flag,
			   cnt_out  => addr
			 );
	last_adr <= X"FD";
	
	INTERVAL_CNT: counter  -- Defines the curve sample interval
	generic map(WID16)
	port map ( clk=>clk, clrn=>nclr_cnt, ena=>'1', load=>'0', updn=>'1',
			   ld_data  => (others => '0'),
			   flag_cnt => intv_clks,
			   flag_out => smp_flag,
			   cnt_out  => open
			 );

	TRIGGER_SYNC: led  -- Syncronizes the start trigger input
	port map(sig_in => Start_Trig, clk => clk, clrn => '1', pls_out => go );
	
	END_CYCLE_SYNC: led  -- Syncronizes the end cycle trigger input
	port map(sig_in => End_Trig, clk => clk, clrn => '1', pls_out => end_cycle );
		
-- State Machine Process definition ---------------------------
	sm_reg: process (clk, nreset)	
	begin
		if(nreset='0')              then CS <= init;
		elsif(clk'event and clk='1')then CS <= NS;
		end if;
	end process; -- sm_reg
	
	sm_combinatorial: process (CS, go, smp_flag, end_flag, end_cycle)
	begin
		-- default value --
		curve_active <= '1'; -- not active in idle
		
		case CS is
			when init   =>
				NS <= idle  ;
				-- outputs --
				curve_active <= '0';
				nclr_cnt <= '0'; -- counter is reset
				nclr_adr <= '0'; -- counter is reset
				inc_adr  <= '0'; -- not incrementing the address pointer
				--------------------
			when idle   =>
				if(go ='1')          then NS <= wait_1  ;
				else                      NS <= idle;
				end if; 
				-- outputs --
				curve_active <= '0';
				nclr_cnt <= '0'; -- counter is reset
				nclr_adr <= '0'; -- counter is reset
				inc_adr <= '0';  -- not incrementing the address pointer
				--------------------
			when wait_1 => -- wait for next sample time
				if(smp_flag ='1')    then NS <= nextadr;
				elsif(end_cycle ='1')then NS <= rst_adr;
				else                      NS <= wait_1;
				end if;
				-- outputs --
				nclr_cnt <= '1'; -- not reset, but waiting
				nclr_adr <= '1';
				inc_adr <= '0';
				--------------------
			when nextadr => 
				if(end_flag ='1')    then NS <= hold;
				elsif(end_cycle ='1')then NS <= rst_adr;
				else                      NS <= wait_1;
				end if;
				-- outputs --
				nclr_cnt <= '0'; -- clear the interval count
				nclr_adr <= '1';
				inc_adr <= '1';  -- enable the addr count
				--------------------
			when hold => -- Hold last address until end of cycle trigger
				if(end_cycle ='1')   then NS <= rst_adr;
				else                      NS <= hold;
				end if;
				-- outputs --
				nclr_cnt <= '1';
				nclr_adr <= '1';
				inc_adr <= '0';
				--------------------
			when rst_adr => -- A two second press will reset the module
				NS <= idle;
				-- outputs --
				nclr_cnt <= '0'; -- clear all counters
				nclr_adr <= '0'; -- clear all counters
				inc_adr <= '0';
				--------------------
		end case; 
				
	end process; -- sm_combinatorial
	
-- Syncronized Outputs ----------------------------
new_gain(11 downto 0) <= not product_2s(23) & product_2s(21 downto 11);

process(clk, nreset, Enable_Gain_Curve, product_2s, base_gain, new_gain, 
		Final_Curve_Gain, curve_active )
begin
	if   (nreset = '0') then
	MDAC_Curve_Out(15 downto 0) <= "0000" & X"800"; -- 12 bit offset binary zero
	
	elsif((Enable_Gain_Curve = '0') OR (curve_active = '0')) then
	MDAC_Curve_Out(15 downto 0) <= "0000" & base_gain(11 downto 0); -- 12 bit offset binary value
										
	elsif rising_edge(clk) then
		if  (base_gain(11 downto 0) > X"800") AND 
			(new_gain < Final_Curve_Gain) then
			MDAC_Curve_Out(15 downto 0) <= "0000" & Final_Curve_Gain(11 downto 0);
			
		elsif(base_gain(11 downto 0) < X"800") AND
			(new_gain > Final_Curve_Gain) then
			MDAC_Curve_Out(15 downto 0) <= "0000" & Final_Curve_Gain(11 downto 0);
			
		else	
			MDAC_Curve_Out(15 downto 0) <= "0000" & new_gain(11 downto 0);
		end if;									 									 
	end if;
end process;


end main;

