Library IEEE;
USE IEEE.std_logic_1164.all;
USE IEEE.std_logic_arith.all;
USE ieee.std_logic_unsigned.all;

-- My Definitions --------------------------------------------------
USE work.pkg_081006.all;  -- my definitions and entities

-- HOW DOES IT WORK ------------------------------------------------
-- 


entity Switch_Off_Delay is
	port(
			clk        	: in std_logic;
			nreset     	: in std_logic;
			Input			: in std_logic;
		 
			Output		: out std_logic		 
		);
end Switch_Off_Delay;
	
architecture main of Switch_Off_Delay is

signal inp			: std_logic;

signal timer		: std_logic_vector(15 downto 0);
signal timer_ena	: std_logic;

	-- State Machine declarations ------------------------------------
	type STATE_TYPE is 
	(idle, switch_on, switch_off, sw_off_delay);
	signal CS, NS  : STATE_TYPE;

-- ## MAIN ##############################################################
begin 

SYNCHRONIZE_INPUT:
process(clk, nreset, Input)
begin
	if   (nreset = '0') then 		inp <= '0';
	elsif rising_edge(clk) then	inp <= Input;
	end if;	
end process;

PROCESS_TIMER: process(clk, nreset, timer_ena, timer)
begin
	if  (nreset = '0') then 		timer <= (others => '0');
	elsif rising_edge(clk) then
		if  (timer_ena = '1') then timer <= timer + 1;
		else								timer <= (others => '0');
		end if;
	end if;
end process;

----------------------------------------------------------
----------------------------------------------------------	
sm_reg: process (clk, nreset)	
begin
	if(nreset='0')              then CS <= idle;
	elsif(clk'event and clk='1')then CS <= NS;
	end if;
end process; -- sm_reg

sm_combinatorial: 
process (CS, timer, inp)
begin
		timer_ena	<= '0';
		
		case CS is
		when idle	=>
			output		<= '0';			
			if (inp = '1') then 		NS <= switch_on;
			else							NS <= idle;
			end if;
			
		when switch_on =>
			output		<= '1';			
			if (inp = '0') then 		NS <= sw_off_delay;
			else							NS <= switch_on;
			end if;
		
		when sw_off_delay =>
			output 		<= '1';
			timer_ena	<= '1';
			if   (inp = '1') then			NS <= switch_on;
			elsif(timer = X"0014") then	NS <= switch_off;
			else									NS <= sw_off_delay;
			end if;
		
		when switch_off =>			
			output 		<= '0';
			if (inp = '1') then 		NS <= switch_on;
			else							NS <= switch_off;
			end if;

		when others =>					NS <= idle;
	end case; 
			
end process;

end main;