Library IEEE;
USE IEEE.std_logic_1164.all;
USE IEEE.std_logic_unsigned.all;

entity Test_Interface_Mux_v2 is 
	port( 	
			clk			: in std_logic;
			nreset		: in std_logic;
			Test_Cntrl	: in std_logic_vector(15 downto 0);
			
			d0			: in std_logic;
			d1			: in std_logic;
			d2			: in std_logic;
			d3			: in std_logic;
			d4			: in std_logic;
			d5			: in std_logic;
			d6			: in std_logic;
			d7			: in std_logic;
			
			d8			: in std_logic;
			d9			: in std_logic;
			d10			: in std_logic;
			d11			: in std_logic;
			d12			: in std_logic;
			d13			: in std_logic;
			d14			: in std_logic;
			d15			: in std_logic;
			
			LED1_in		: in std_logic;
			LED2_in		: in std_logic;
			LED3_in		: in std_logic;
			LED4_in		: in std_logic;
			
			HOLD1_in	: in std_logic;
			HOLD2_in	: in std_logic;
			
			DOUT1_in	: in std_logic;
			DOUT2_in	: in std_logic;
			DOUT3_in	: in std_logic;
			DOUT4_in	: in std_logic;
			
			alt_DOUT1_in	: in std_logic;
			alt_DOUT2_in	: in std_logic;
			alt_DOUT3_in	: in std_logic;
			alt_DOUT4_in	: in std_logic;
			           
			Sel_DAC_in		: in std_logic;
			Sel_EXT_in		: in std_logic;
			Sel_MDAC1_in	: in std_logic;
			Sel_MDAC2_in	: in std_logic;
			
			DAC_Test_In		: in std_logic_vector(15 downto 0);
			DAC_Tst_Strb_In	: in std_logic;
			DAC_1A_DATA_In	: in std_logic_vector(15 downto 0);
			DAC_1A_Strb_In	: in std_logic;
			DAC_2A_DATA_In	: in std_logic_vector(15 downto 0);
			DAC_2A_Strb_In	: in std_logic;
			DAC_1B_DATA_In	: in std_logic_vector(15 downto 0);
			DAC_1B_Strb_In	: in std_logic;
			DAC_2B_DATA_In	: in std_logic_vector(15 downto 0);
			DAC_2B_Strb_In	: in std_logic;
			
			dout		: out std_logic_vector(15 downto 0);
			LED1_out	: out std_logic;
			LED2_out	: out std_logic;
			LED3_out	: out std_logic;
			LED4_out	: out std_logic;
			     
			HOLD1_out	: out std_logic;
			HOLD2_out	: out std_logic;
			      
			DOUT1_out	: out std_logic;
			DOUT2_out	: out std_logic;
			DOUT3_out	: out std_logic;
			DOUT4_out	: out std_logic;
			
			Sel_DAC_out		: out std_logic;
			Sel_EXT_out		: out std_logic;
			Sel_MDAC1_out	: out std_logic;
			Sel_MDAC2_out	: out std_logic;
			
			DAC_1_DATA		: out std_logic_vector(15 downto 0);
			DAC_1_Strb		: out std_logic;

			DAC_2_DATA		: out std_logic_vector(15 downto 0);			
			DAC_2_Strb		: out std_logic;
			
			Diag_Select		: out std_logic_vector(3 downto 0);
			nTestMode		: out std_logic;
			
			Ena_ZC_Start 	: out std_logic;
			Reset_All		: out std_logic;
			Save_All		: out std_logic
			      		      
		);
	end Test_Interface_Mux_v2;
	
architecture main of Test_Interface_Mux_v2 is	
             
signal interface_mode		: std_logic_vector(1 downto 0);
signal Select_Alt_DAC		: std_logic;

           
begin
	
--=====================================================================================	
	DATA_BUFFER: 
	-- This 16 bit word is filled with digital signals for reading back over USB.
	process(clk, nreset, d0, d1, d2, d3, d4, d5, d6, d7, 
						 d8, d9, d10, d12, d13, d14, d15)
	begin
		if   (nreset = '0') then dout <= (others => '0');
		elsif rising_edge(clk) then
			dout(15 downto 0) <= d15&d14&d13&d12&d11&d10&d9&d8&d7&d6&d5&d4&d3&d2&d1&d0;
		end if;
	end process;
	
--=====================================================================================
-- OPTION SELECTION SIGNALS
	interface_mode 	<= Test_Cntrl(13 downto 12);
	
	Ena_ZC_Start	<= Test_Cntrl(14);
	Select_Alt_DAC	<= Test_Cntrl(15);
	
--=====================================================================================
	process(Select_Alt_DAC, interface_mode,
			DAC_Test_In, DAC_Tst_Strb_In, DAC_1A_DATA_In, DAC_1A_Strb_In,	
			DAC_2A_DATA_In, DAC_2A_Strb_In,	DAC_1B_DATA_In,	DAC_1B_Strb_In,	
			DAC_2B_DATA_In,	DAC_2B_Strb_In)
			
	begin
		if (interface_mode = "01") then
			DAC_1_DATA <= DAC_Test_In(15 downto 0);
			DAC_1_Strb <= DAC_Tst_Strb_In;

			DAC_2_DATA <= DAC_Test_In(15 downto 0);
			DAC_2_Strb <= DAC_Tst_Strb_In;
			
		elsif(Select_Alt_DAC = '1') then               
			DAC_1_DATA <= DAC_1B_DATA_In(15 downto 0);
			DAC_1_Strb <= DAC_1B_Strb_In;						

			DAC_2_DATA <= DAC_2B_DATA_In(15 downto 0);
			DAC_2_Strb <= DAC_2B_Strb_In;						

		else
			DAC_1_DATA <= DAC_1A_DATA_In(15 downto 0);
			DAC_1_Strb <= DAC_1A_Strb_In;						

			DAC_2_DATA <= DAC_2A_DATA_In(15 downto 0);
			DAC_2_Strb <= DAC_2A_Strb_In;						

		end if;             
	end process;
	
--=====================================================================================
	INTERFACE_SIGNAL_MUX:
	process(interface_mode, Test_Cntrl, 
			LED1_in, LED2_in, LED3_in, LED4_in, 
			HOLD1_in, HOLD2_in, 
			DOUT1_in, DOUT2_in, DOUT3_in, DOUT4_in, 
			Sel_DAC_in, Sel_EXT_in, Sel_MDAC1_in, Sel_MDAC2_in,
			alt_DOUT1_in, alt_DOUT2_in, alt_DOUT3_in, alt_DOUT4_in)
	begin
		-- DEFAULTS --
				LED1_out		<= LED1_in;		
			    LED2_out		<= LED2_in;		
			    LED3_out		<= LED3_in;		
			    LED4_out		<= LED4_in;		
			                       
			    HOLD1_out		<= HOLD1_in;	
			    HOLD2_out		<= HOLD2_in;
			                       
			    DOUT1_out		<= DOUT1_in;	
			    DOUT2_out		<= DOUT2_in;	
			    DOUT3_out		<= DOUT3_in;	
			    DOUT4_out		<= DOUT4_in;	
			                       
			    Sel_DAC_out		<= Sel_DAC_in;	
			    Sel_EXT_out		<= Sel_EXT_in;	
			    Sel_MDAC1_out	<= Sel_MDAC1_in;
			    Sel_MDAC2_out	<= Sel_MDAC2_in;
			    							       
			    Diag_Select(3 downto 0) <= "1111";
			    
			    Reset_All		<= '0';	
			    Save_All		<= Test_Cntrl(10);
			      
			    nTestMode		<= '1'; -- ACTIVE LOW
			    
		case interface_mode is
			when "00" =>  -- DEFAULT MODE --

			when "01" => 
				LED1_out		<= Test_Cntrl(6);		
			    LED2_out		<= Test_Cntrl(7);	
			    LED3_out		<= Test_Cntrl(8);	
			    LED4_out		<= Test_Cntrl(9);	
			                       
			    HOLD1_out		<= Test_Cntrl(4);	
			    HOLD2_out		<= Test_Cntrl(5);	
			                       
			    DOUT1_out		<= Test_Cntrl(6);	
			    DOUT2_out		<= Test_Cntrl(7);	
			    DOUT3_out		<= Test_Cntrl(8);	
			    DOUT4_out		<= Test_Cntrl(9);	
			                                 
			    Sel_DAC_out		<= Test_Cntrl(0);	
			    Sel_EXT_out		<= Test_Cntrl(1);	
			    Sel_MDAC1_out	<= Test_Cntrl(3);	
			    Sel_MDAC2_out	<= Test_Cntrl(2);
			    	
			    Diag_Select(3 downto 0) <= "1111";
			      
			    Reset_All		<= Test_Cntrl(10);	-- CAUTION THIS WILL REMOVE US FROM TEST MODE.
													-- Reset All will load all registers from EEPROM
			    nTestMode		<= '0';
			                     
			when "10" => 
			    --enable the Diagnositic Port in Normal Mode			      
			    Diag_Select(3 downto 0) <= Test_Cntrl(3 downto 0);
			    DOUT1_out		<= alt_DOUT1_in;	
			    DOUT2_out		<= alt_DOUT2_in;	
			    DOUT3_out		<= alt_DOUT3_in;	
			    DOUT4_out		<= alt_DOUT4_in;	
			      
			when others => 
			    
			end case;	
	
	end process;

end;