Library IEEE;
USE IEEE.std_logic_1164.all;
USE IEEE.std_logic_unsigned.all;
-- My Definitions --------------------------------------------------
USE work.pkg_081006.all;  -- my definitions and entities
--------------------------------------------------------------------
-- OPTIONS
-- Test_Cntrl(15 downto 0)
-- 	[15 ... 12] => | Test Filter 1 | Test Filter 2 | Use Alt DO | Production Test |
--    [11 ... 8 ] =>   Diagnostic Port J14 MUX Select (3 downto 0) 
--    [ 7 ... 4 ] =>   Aux DAC 2 MUX Select (3 downto 0)
--    [ 3 ... 0 ] =>   Aux DAC 1 MUX Select (3 downto 0)


entity Test_Interface_Mux_v3 is 
	port( 	
			clk					: in std_logic;
			nreset				: in std_logic;
			Test_Cntrl			: in std_logic_vector(15 downto 0);
			            		
			d0						: in std_logic;
			d1						: in std_logic;
			d2						: in std_logic;
			d3						: in std_logic;
			d4						: in std_logic;
			d5						: in std_logic;
			d6						: in std_logic;
			d7						: in std_logic;
			
			d8						: in std_logic;
			d9						: in std_logic;
			d10					: in std_logic;
			d11					: in std_logic;
			d12					: in std_logic;
			d13					: in std_logic;
			d14					: in std_logic;
			d15					: in std_logic;
			
			LED1_in				: in std_logic;
			LED2_in				: in std_logic;
			LED3_in				: in std_logic;
			LED4_in				: in std_logic;
			
			HOLD1_in				: in std_logic;
			
			DOUT1_in				: in std_logic;
			DOUT2_in				: in std_logic;
			DOUT3_in				: in std_logic;
			DOUT4_in				: in std_logic;
			
			alt_DOUT1_in		: in std_logic;
			alt_DOUT2_in		: in std_logic;
			alt_DOUT3_in		: in std_logic;
			alt_DOUT4_in		: in std_logic;
			           
			Sel_DAC_in			: in std_logic;
			Sel_EXT_1_in		: in std_logic;
			Sel_PD_FILTER_in	: in std_logic;
			Sel_EXT_2_in		: in std_logic;
						
			Data_out				: out std_logic_vector(15 downto 0);
			LED1_out				: out std_logic;
			LED2_out				: out std_logic;
			LED3_out				: out std_logic;
			LED4_out				: out std_logic;
			     
			HOLD1_out			: out std_logic;
			      
			DOUT1_out			: out std_logic;
			DOUT2_out			: out std_logic;
			DOUT3_out			: out std_logic;
			DOUT4_out			: out std_logic;
			
			Sel_DAC_out			: out std_logic;
			Sel_EXT_1_out		: out std_logic;
			Sel_PD_FILTER_out	: out std_logic;
			Sel_EXT_2_out		: out std_logic;
			
			DAC1_Select			: out std_logic_vector(3 downto 0);
			DAC2_Select			: out std_logic_vector(3 downto 0);
			Diag_Select			: out std_logic_vector(3 downto 0);
			nTestMode			: out std_logic;
			
			Reset_All			: out std_logic;
			Save_All				: out std_logic
			      		      
		);
	end Test_Interface_Mux_v3;
	
architecture main of Test_Interface_Mux_v3 is	

signal use_alt_DO				: std_logic; 
signal prod_test_mode		: std_logic;            
signal do_reset_all			: std_logic;
signal usb_save_all			: std_logic;

           
begin
	
--=====================================================================================	
	DATA_BUFFER: 
	-- This 16 bit word is filled with digital signals for reading back over USB.
	process(clk, nreset, d0, d1, d2, d3, d4, d5, d6, d7, 
						 d8, d9, d10, d12, d13, d14, d15)
	begin
		if   (nreset = '0') then Data_out <= (others => '0');
		elsif rising_edge(clk) then
			Data_out(15 downto 0) <= d15&d14&d13&d12&d11&d10&d9&d8&d7&d6&d5&d4&d3&d2&d1&d0;
		end if;
	end process;
	
--=====================================================================================
-- Test_Cntrl(15 downto 0)
-- 	[15 ... 12] => | reserved | reserved | Use Alt DO | Production Test |
--    [11 ... 8 ] =>   Diagnostic Port J14 MUX Select (3 downto 0) 
--    [ 7 ... 4 ] =>   Aux DAC 2 MUX Select (3 downto 0)
--    [ 3 ... 0 ] =>   Aux DAC 1 MUX Select (3 downto 0)

	--reserved						 <= Test_Cntrl(15);
	--reserved						 <= Test_Cntrl(14);
	use_alt_DO						 <= Test_Cntrl(13);
	prod_test_mode					 <= Test_Cntrl(12);
	Diag_Select(3 downto 0) 	 <= Test_Cntrl(11 downto 8);
	DAC2_Select(3 downto 0) 	 <= Test_Cntrl(7 downto 4); -- these alias as DO in test mode
	DAC1_Select(3 downto 0) 	 <= Test_Cntrl(3 downto 0); -- these alias as DO in test mode

--=====================================================================================
	RESET_ALL_REGISTERS: led
port map(sig_in => NOT prod_test_mode, clk => clk, clrn => nreset, pls_out => do_reset_all);

	Reset_All <= do_reset_all; -- set when leaving the production test mode

	SAVE_ALL_REGISTERS: led
port map(sig_in => use_alt_DO, clk => clk, clrn => nreset, pls_out => usb_save_all);

	Save_All <= usb_save_all AND prod_test_mode; -- alternate use of Use_Alt_DO when in test mode.  Must toggle High.

--=====================================================================================
	PRODUCTION_TEST_SELECT:
	process(Test_Cntrl, prod_test_mode,
			LED1_in, LED2_in, LED3_in, LED4_in, HOLD1_in,
			Sel_DAC_in, Sel_EXT_1_in, Sel_PD_FILTER_in, Sel_EXT_2_in,
			use_alt_DO, DOUT1_in, DOUT2_in, DOUT3_in, DOUT4_in, 
			alt_DOUT1_in, alt_DOUT2_in, alt_DOUT3_in, alt_DOUT4_in)
			
	begin
		if (prod_test_mode = '0') then -- normal mode 
				 LED1_out		<= LED1_in;		
			    LED2_out		<= LED2_in;		
			    LED3_out		<= LED3_in;		
			    LED4_out		<= LED4_in;		
			                       
					if   (use_alt_DO = '1') then
							 DOUT1_out		<= alt_DOUT1_in;	
							 DOUT2_out		<= alt_DOUT2_in;	
							 DOUT3_out		<= alt_DOUT3_in;	
							 DOUT4_out		<= alt_DOUT4_in;	
					else
							 DOUT1_out		<= DOUT1_in;	
							 DOUT2_out		<= DOUT2_in;	
							 DOUT3_out		<= DOUT3_in;	
							 DOUT4_out		<= DOUT4_in;	
					end if;
			                       
			    Sel_DAC_out			<= Sel_DAC_in;	
			    Sel_EXT_1_out			<= Sel_EXT_1_in;	
			    Sel_PD_FILTER_out	<= Sel_PD_FILTER_in;
			    Sel_EXT_2_out			<= Sel_EXT_2_in;
			                       
			    HOLD1_out		<= HOLD1_in;	
			    							       
			    nTestMode		<= '1'; -- ACTIVE LOW
				 
		else -- test mode
				 LED1_out		<= Test_Cntrl(4);		
			    LED2_out		<= Test_Cntrl(5);	
			    LED3_out		<= Test_Cntrl(6);	
			    LED4_out		<= Test_Cntrl(7);	
			                       
			    DOUT1_out		<= Test_Cntrl(4);	
			    DOUT2_out		<= Test_Cntrl(5);	
			    DOUT3_out		<= Test_Cntrl(6);	
			    DOUT4_out		<= Test_Cntrl(7);	
			                                 
			    Sel_DAC_out			<= Test_Cntrl(0);	
			    Sel_EXT_1_out			<= Test_Cntrl(1);	
			    Sel_PD_FILTER_out	<= Test_Cntrl(3);	
			    Sel_EXT_2_out			<= Test_Cntrl(2);
				 
			    HOLD1_out				<= Test_Cntrl(4);	-- same as DOUT1
			    	
			    nTestMode		<= '0';			                     
			end if;	
	end process;
	
end;