Library IEEE;
USE IEEE.std_logic_1164.all;
USE IEEE.std_logic_arith.all;
USE ieee.std_logic_unsigned.all;

-- My Definitions --------------------------------------------------
USE work.pkg_081006.all;  -- my definitions and entities


--Version 4 breaks the EXTERNAL / MADC1 Switch logic out from the 
-- Accel_PL_Ena and Start / Stop logic.

--Version 5 adds an adjustable delay to Hold_for_MIPL

entity Test_Triggers is
	port(clk        	: in std_logic;
		 nreset     	: in std_logic;
		 Beam_Gate		: in std_logic;
		 Accel_Gate		: in std_logic;
		 
		 Beam_Gate_Trig_1 : out std_logic;
 		 Beam_Gate_Trig_2 : out std_logic

		);
end Test_Triggers;
	
architecture main of Test_Triggers is

signal syncd_Accel_Gate	: std_logic;
signal syncd_Beam_Gate 	: std_logic;

signal timer			: std_logic_vector(23 downto 0);
signal timer_ena		: std_logic;

signal timer2			: std_logic_vector(23 downto 0);
signal timer2_ena		: std_logic;

signal go				:std_logic;

	-- State Machine declarations ------------------------------------
	type STATE_TYPE is 
	(idle, beam_gate_on, beam_gate_off, clear_timer2a, beam_gate_failed, 
	 clear_timer2b, beam_cycle_failed, done, final_pause);
	signal CS, NS  : STATE_TYPE;

-- ## MAIN ##############################################################
begin 
EDGE_ON_BEAMGATE: led
port map(sig_in => Beam_Gate, clk => clk, clrn => nreset, pls_out => go);



process(clk, nreset, Beam_Gate, Accel_Gate)
begin
	if (nreset = '0') then 
		syncd_Accel_Gate	<= '0';
		syncd_Beam_Gate		<= '0';
	elsif rising_edge(clk) then
		syncd_Accel_Gate	<= Accel_Gate;
		syncd_Beam_Gate		<= Beam_Gate;
	end if;
end process;

PROCESS_TIMER: process(clk, nreset, timer_ena, timer)
begin
	if  (nreset = '0') then 	timer <= (others => '0');
	elsif rising_edge(clk) then
		if  (timer_ena = '0') then 	timer <= (others => '0');
		else						timer <= timer + 1;
		end if;
	end if;
end process;

PROCESS_TIMER2: process(clk, nreset, timer2_ena, timer2)
begin
	if  (nreset = '0') then 	timer2 <= (others => '0');
	elsif rising_edge(clk) then
		if  (timer2_ena = '0') then 	timer2 <= (others => '0');
		else						timer2 <= timer2 + 1;
		end if;
	end if;
end process;

----------------------------------------------------------
----------------------------------------------------------	
sm_reg: process (clk, nreset)	
begin
	if(nreset='0')              then CS <= idle;
	elsif(clk'event and clk='1')then CS <= NS;
	end if;
end process; -- sm_reg

sm_combinatorial: 
process (CS, go, timer, timer2, syncd_Beam_Gate, syncd_Accel_Gate)
begin
	-- default outputs --
	timer_ena 		<= '1'; -- not enabled only when in idle state
	timer2_ena 		<= '0'; -- not enabled by default

	Beam_Gate_Trig_1 <= '0';
	Beam_Gate_Trig_2 <= '0';
		
	case CS is
		when idle   =>
			timer_ena		<= '0';  
			if (go = '1') then 					NS <= beam_gate_on;	
			else								NS <= idle;
			end if;

		when beam_gate_on =>
			
			timer_ena 		<= '1'; -- time from first occurance of beam gate High
			timer2_ena		<= '0'; -- timer 2 is cleared
			if   (timer > X"28A640") then			NS <= done; -- 33.3 ms time out
			elsif(syncd_Beam_Gate = '0') then		
				if   (timer > X"04E200") then		NS <= clear_timer2b; -- 4 ms
				else								NS <= beam_gate_off;
				end if;
			else									NS <= beam_gate_on;
			end if;

		when beam_gate_off =>
			
			Beam_Gate_Trig_1 <= '0';
			Beam_Gate_Trig_2 <= '0';
			
			timer_ena		<= '1';
			timer2_ena		<= '1';
			if (timer2 < X"013880") then	-- beam gate off for less than 1 ms		
				if(syncd_Beam_Gate = '1') then		NS <= clear_timer2a; 
				elsif(timer > X"271000") then		NS <= done; -- 32 ms time out
				else								NS <= beam_gate_off;
				end if;
			else									NS <= clear_timer2b;
			end if;
		
		when clear_timer2a => 						NS <= beam_gate_failed;
			timer_ena	<= '1';
			timer2_ena	<= '0';
		
		when beam_gate_failed =>
			
			Beam_Gate_Trig_1 <= '1';
			Beam_Gate_Trig_2 <= '0';
			
			timer_ena		<= '1';
			timer2_ena 		<= '1';
			if   (timer2 > X"013880") then			NS <= beam_gate_on; -- 1 ms minimum pulse width
--				if(syncd_Beam_Gate = '1') then		NS <= beam_gate_on; 
--				elsif(timer > X"298100") then		NS <= idle; -- 34 ms time out
--				else								NS <= beam_gate_failed;
--				end if;
			else									NS <= beam_gate_failed;
			end if;

		when clear_timer2b => 						NS <= beam_cycle_failed;
			timer_ena	<= '1';
			timer2_ena	<= '0';
		
		when beam_cycle_failed =>
			
			Beam_Gate_Trig_1 <= '0';
			Beam_Gate_Trig_2 <= '1';
			
			timer_ena		<= '1';
			timer2_ena 		<= '1';
			if   (timer2 > X"013880") then			NS <= idle;-- 1 ms minimum pulse width
			else									NS <= beam_cycle_failed;
			end if;
		
		when done =>
			timer2_ena 		<= '0';
			if   (syncd_Beam_Gate = '0') then		NS <= final_pause;
			else									NS <= done;
			end if;

		when final_pause =>  -- this avoids rough bits when Beam Gate shuts off
			timer_ena		<= '0';
			timer2_ena 		<= '1';
			if   (timer2 > X"0C3500") then			NS <= idle;-- 10 ms minimum pulse width
			else									NS <= final_pause;
			end if;
		
		when others =>							NS <= idle;
	end case; 
			
end process;

end main;